# SPDX-FileCopyrightText: 2025 Florian Obersteiner <f.obersteiner@kit.edu>
#
# SPDX-License-Identifier: GPL-3.0-or-later

# -*- coding: utf-8 -*-
from pathlib import Path

import dash_bootstrap_components as dbc
import pandas as pd
import plotly.graph_objects as go
from dash import Dash
from dash.dependencies import Input, Output

from src.config import DotDict
from src.data import loader
from src.layout import create_layout

appconfig = DotDict.from_toml_file("./config/env.toml")
varconfig = pd.read_csv("./config/parms_units.csv", sep=";")

# fmt: off
app = Dash(
    appconfig.TITLE,
    requests_pathname_prefix=appconfig.REQUESTS_PATH_PREFIX,
    external_stylesheets=[dbc.themes.BOOTSTRAP, dbc.icons.FONT_AWESOME],
)
# fmt: on

app.title = appconfig.TITLE
app.css.config.serve_locally = True
app.scripts.config.serve_locally = True

data = loader(Path("./testdata").glob("*.nc"), appconfig, True)  # type: ignore

app.layout = create_layout(data, appconfig)
server = app.server  # for deployment via gunicorn / WSGI server


@app.callback(
    [Output("fig-map", "figure"), Output("fig-ts", "figure")],
    [Input("flight-dropdown", "value"), Input("variable-dropdown", "value")],
)
def update_map(selected_flight: int, selected_variable: str):
    filtered_df = data[data["flight_number"] == selected_flight]
    valid_indices = filtered_df[selected_variable].notna()
    unit = varconfig.loc[selected_variable == varconfig.VNAME, "Unit"].values[0]

    fig_map = go.Figure(
        go.Scattermap(
            lon=filtered_df.loc[valid_indices, "lon"],
            lat=filtered_df.loc[valid_indices, "lat"],
            mode="markers",
            marker={
                "color": filtered_df.loc[valid_indices, selected_variable],
                "colorscale": "Rainbow",
                "colorbar": {
                    "x": 0.995,
                    "title": {"text": f"{selected_variable} {unit}", "side": "right"},
                },
                "showscale": True,
            },
            connectgaps=False,
            hoverinfo="text",
            hovertext=[f"{v:.1f}" for v in filtered_df[selected_variable]],
            name="Flight Path",
        )
    )
    fig_map.add_trace(
        go.Scattermap(
            lat=[filtered_df.lat.iloc[0]],
            lon=[filtered_df.lon.iloc[0]],
            mode="markers",
            marker={"size": 15, "symbol": "airport", "allowoverlap": True},
            text=["departure airport"],
            hoverinfo="text",
            name="departure",
        )
    )

    fig_map.update_layout(
        map={
            "style": "outdoors",
            "zoom": 1.7,
            "center": {"lat": filtered_df.lat.mean(), "lon": filtered_df.lon.mean()},
        },
        margin={"l": 0, "r": 10, "t": 0, "b": 0},
        showlegend=False,
    )

    fig_ts = go.Figure()
    fig_ts.add_trace(
        go.Scatter(
            x=filtered_df.index[valid_indices & ~filtered_df["BB_flag"]],
            y=filtered_df.loc[valid_indices & ~filtered_df["BB_flag"], selected_variable],
            mode="markers",
            marker={
                "size": 3,
                "color": "blue",
                "showscale": False,
            },
            showlegend=False,
        )
    )
    fig_ts.add_trace(
        go.Scatter(
            x=filtered_df.index[valid_indices & filtered_df["BB_flag"]],
            y=filtered_df.loc[valid_indices & filtered_df["BB_flag"], selected_variable],
            mode="markers",
            marker={
                "size": 3,
                "color": "red",
                "showscale": False,
            },
            name="flagged 'biomass burning'",
        )
    )

    fig_ts.update_layout(
        paper_bgcolor="#e8e8e8",
        margin={"l": 45, "r": 10, "t": 20, "b": 50},
        legend={
            "x": 0.01,
            "y": 0.99,
            "xanchor": "left",
            "yanchor": "top",
            "bgcolor": "rgba(255, 255, 255, 0.7)",
            "bordercolor": "rgba(0, 0, 0, 0.2)",
            "borderwidth": 1,
        },
    )

    fig_ts.update_xaxes(title="UTC")
    fig_ts.update_yaxes(title=f"{selected_variable} {unit}")

    return [fig_map, fig_ts]


if __name__ == "__main__":
    app.run(debug=appconfig.DEBUG)  # type: ignore
