import tomllib


class DotDict(dict):
    """Wrapper class around dict for dot-notation access to dictionary keys."""

    def __init__(self, *args, **kwargs):
        """Initialize DotDict and recursively convert nested dictionaries."""
        super().__init__(*args, **kwargs)
        self._convert_nested()

    def _convert_nested(self, path=""):
        """Convert nested dictionaries to DotDict instances and validate keys."""
        for key, value in list(self.items()):
            # Validate keys
            if not isinstance(key, str):
                continue

            # Check for invalid attribute names
            if not key.isidentifier():
                print(
                    f"Warning: '{path}{key}' is not a valid Python identifier and cannot be accessed with dot notation"
                )

            # Check for reserved dictionary method names
            if key in dir(dict) and key not in ("__class__", "__dict__"):
                print(f"Warning: '{path}{key}' overrides a built-in dictionary method")

            # Recursively convert nested dictionaries
            if isinstance(value, dict) and not isinstance(value, DotDict):
                self[key] = DotDict(value)
                self[key]._convert_nested(f"{path}{key}.")
            elif isinstance(value, list):
                # Handle lists and recursively process any dictionaries within them
                self[key] = self._process_list_items(value, f"{path}{key}")

    def _process_list_items(self, items, path):
        """Recursively process items in a list, converting any dicts to DotDict."""
        processed_items = []
        for i, item in enumerate(items):
            if isinstance(item, dict) and not isinstance(item, DotDict):
                dot_dict_item = DotDict(item)
                dot_dict_item._convert_nested(f"{path}[{i}].")
                processed_items.append(dot_dict_item)
            elif isinstance(item, list):
                # Handle nested lists recursively
                processed_items.append(self._process_list_items(item, f"{path}[{i}]"))
            else:
                processed_items.append(item)
        return processed_items

    @classmethod
    def from_toml_data(cls, toml_data):
        """Create a DotDict instance from TOML data."""
        return cls(toml_data)

    @classmethod
    def from_toml_file(cls, file_path):
        """Load a TOML file into a DotDict instance."""
        with open(file_path, "rb") as f:
            config_data = tomllib.load(f)

        return cls(config_data)

    def __getattr__(self, key):
        """Enable dot notation access for dictionary keys."""
        if key in self:
            return self[key]
        raise AttributeError(f"'{self.__class__.__name__}' object has no attribute '{key}'")

    def __setattr__(self, key, value):
        """Enable dot notation assignment for dictionary keys."""
        # Convert dictionaries to DotDict when setting attributes
        if isinstance(value, dict) and not isinstance(value, DotDict):
            value = DotDict(value)
        elif isinstance(value, list):
            # Process lists when setting them as attributes
            value = self._process_list_items(value, key)
        self[key] = value

    def __delattr__(self, key):
        """Enable dot notation deletion for dictionary keys."""
        if key in self:
            del self[key]
        else:
            raise AttributeError(f"'{self.__class__.__name__}' object has no attribute '{key}'")
